<?php
if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

class HAI_API_Action_After_Submit extends \ElementorPro\Modules\Forms\Classes\Action_Base
{

    /**
     * Get action name.
     *
     * Retrieve Helios action name.
     *
     * @since 1.0.0
     * @access public
     * @return string
     */
    public function get_name()
    {
        return 'heliosai';
    }

    /**
     * Get action label.
     *
     * Retrieve Helios action label.
     *
     * @since 1.0.0
     * @access public
     * @return string
     */
    public function get_label()
    {
        return esc_html__('HeliosAI', 'heliosai-connect');
    }

    /**
     * Run action.
     *
     * Helios an external server after form submission.
     *
     * @since 1.0.0
     * @access public
     * @param \ElementorPro\Modules\Forms\Classes\Form_Record  $record
     * @param \ElementorPro\Modules\Forms\Classes\Ajax_Handler $ajax_handler
     */
    public function run($record, $ajax_handler)
    {

        wp_remote_post(
            'https://api.example.com/',
            [
                'method' => 'GET',
                'headers' => [
                    'Content-Type' => 'application/json',
                ],
                'body' => wp_json_encode([
                    'site' => get_home_url(),
                    'action' => 'Form submitted',
                ]),
                'httpversion' => '1.0',
                'timeout' => 60,
            ]
        );

    }

    /**
     * Register action controls.
     *
     * Helios action has no input fields to the form widget.
     *
     * @since 1.0.0
     * @access public
     * @param \Elementor\Widget_Base $widget
     */
    public function register_settings_section($widget)
    {}

    /**
     * On export.
     *
     * Helios action has no fields to clear when exporting.
     *
     * @since 1.0.0
     * @access public
     * @param array $element
     */
    public function on_export($element)
    {}

}