<?php
namespace HAI\Includes;

// Hírlevél feliratkozás action -- Heisenberg

/**
 * Elementor form HeliosAI Newsletter addon.
 *
 * Custom Elementor form action which adds newsletter subscribers to HeliosAI system.
 *
 * @since 1.0.0
 */

use HAI\Includes\HAI_Connect_helios;

class HAI_Elementor_Form_Newsletter extends \ElementorPro\Modules\Forms\Classes\Action_Base
{

    /**
     * Get action name.
     *
     * @since 1.0.0
     * @access public
     * @return string
     */
    public function get_name()
    {
        return 'heliosai_newsletter';
    }

    /**
     * Get action label.
     *
     * @since 1.0.0
     * @access public
     * @return string
     */
    public function get_label()
    {
        return esc_html__('HeliosAI Newsletter', 'heliosai-connect');
    }

    /**
     * Register action controls.
     *
     * Add input fields to allow the user to customize the action settings.
     *
     * @since 1.0.0
     * @access public
     * @param \Elementor\Widget_Base $widget
     */
    public function register_settings_section($widget)
    {

        $widget->start_controls_section(
            'section_register_to_helios',
            [
                'label' => esc_html__('HeliosAI newsletter', 'heliosai-connect'),
                'condition' => [
                    'submit_actions' => $this->get_name(),
                ],
            ]
        );

        $widget->add_control(
            'hai_newsletter_email',
            [
                'label' => esc_html__('Email Field ID', 'heliosai-connect'),
                'type' => \Elementor\Controls_Manager::TEXT,
            ]
        );

        $postfields = [];
        $postfields['apikey'] = get_option( 'hai_api_token' );
        $res = ( new HAI_Connect_helios('getNewsletterLists') )->connect($postfields);
        $options = [];
        foreach ($res as $row) {
            $options[$row->id] = $row->name;
        }

        $widget->add_control(
            'hai_newsletter_group',
            [
                'label' => __('Newsletter Group <i class="eicon-search" id="hai-connect-reload-newsletter" style="cursor: pointer;"></i>', 'heliosai-connect'),
                'type' => \Elementor\Controls_Manager::SELECT,
                'options' => $options,
            ]
        );

        $widget->end_controls_section();
    }

    /**
     * Run action.
     *
     * Runs the Register to Helios action after form submission.
     *
     * @since 1.0.0
     * @access public
     * @param \ElementorPro\Modules\Forms\Classes\Form_Record  $record
     * @param \ElementorPro\Modules\Forms\Classes\Ajax_Handler $ajax_handler
     */
    public function run($record, $ajax_handler)
    {

        $settings = $record->get('form_settings');

        //  Make sure that there is an email and a password field set in elementor.
        if (empty($settings['hai_newsletter_email']) || empty($settings['hai_newsletter_group'])) {
            $ajax_handler->add_error_message(esc_html__("Valami hiba történt, kérjük próbálja újra később!", 'heliosai-connect'));
            $ajax_handler->add_error('', '');
            return;
        }

        // Get submitted form data.
        $raw_fields = $record->get('fields');

        // Normalize form data.
        $fields = [];
        foreach ($raw_fields as $id => $field) {
            $fields[$id] = $field['value'];
        }

        // Make sure the user entered an email and password.
        if (empty($fields[$settings['hai_newsletter_email']])) {
            $ajax_handler->add_error($settings['hai_newsletter_email'], esc_html__("Nem lett email cím megadva!", 'heliosai-connect'));
            return;
        }

        // Request data based on the helios API
        $postfields = [
            'apikey' => get_option( 'hai_api_token' ),
            'email' => $fields[$settings['hai_newsletter_email']],
            'group_id' => $settings['hai_newsletter_group']
        ];

        // Send the request.
        $res = ( new HAI_Connect_helios('newNewsletterSubscriber') )->connect($postfields);

        if ($res !== true) {

            $ajax_handler->add_error($settings['hai_newsletter_email'], esc_html__("Ezzel az email címmel már feliratkoztak a hírlevélre!", 'heliosai-connect'));
            $ajax_handler->add_error_message('Ezzel az email címmel már feliratkoztak a hírlevélre!');
            return;
        }
    }

    /**
     * On export.
     *
     * Clears Register to helios form settings/fields when exporting.
     *
     * @since 1.0.0
     * @access public
     * @param array $element
     */
    public function on_export($element)
    {

        unset(
            $element['hai_newsletter_email'],
            $element['hai_newsletter_group']
        );

        return $element;
    }
}