<?php

namespace HAI\Includes;

use HAI\Includes\HAI_Connect_helios;

class HAI_Functions {

    public function __construct() {

        add_filter( 'woocommerce_related_products', [$this, 'add_related_products'] );
        
        // add_action( 'woocommerce_process_shop_order_meta', [ $this, 'update_order' ], 10, 1 );
        add_action( 'woocommerce_thankyou', [ $this, 'update_order' ], 10, 1 );
        add_action( 'woocommerce_order_status_changed', [ $this, 'update_order' ], 10, 1 );
        add_action( 'woocommerce_before_calculate_totals', [ $this, 'set_hai_price' ], 20, 1 );
        
        // Added by bence
        add_action('woocommerce_checkout_create_order_line_item', [$this, 'save_as_custom_order_item_meta_data'], 10, 4);
        add_filter('woocommerce_order_item_get_formatted_meta_data', [$this, 'unset_specific_order_item_meta_data'], 10, 2);

        add_action('template_redirect', [$this, 'recreate_cart']);

        add_filter( 'woocommerce_billing_fields', [$this, 'hai_move_checkout_email_field'] );

        add_action('woocommerce_review_order_before_submit', [$this, 'hai_newsletter_checkbox']);
        add_action('woocommerce_checkout_update_order_meta', [$this, 'hai_join_to_newsletter_on_order']);

    }

    public function hai_join_to_newsletter_on_order($order_id){
        if (isset($_POST['hai_newsletter']) && $_POST['hai_newsletter'] == 1) {
            // Retrieve the order object to get the billing email
            $order = wc_get_order($order_id);
            $billing_email = $order->get_billing_email();

            // Request data based on the helios API
            $postfields = [
                'apikey' => get_option( 'hai_api_token' ),
                'email' => $billing_email,
                'group_id' => get_option('hai_newsletter_default_group')
            ];

        // Send the request.
        $res = ( new HAI_Connect_helios('newNewsletterSubscriber') )->connect($postfields);
        }
    }

    public function hai_newsletter_checkbox(){
        if(!get_option('hai_newsletter_is_on') || !get_option('hai_newsletter_default_group')){
            return;
        }
        echo '<div id="hai_newsletter">';
        woocommerce_form_field('hai_newsletter', array(
            'type'          => 'checkbox',
            'class'         => array('hai_newsletter_checkbox'),
            'label'         => __(get_option('hai_newsletter_checkbox_text') ? get_option('hai_newsletter_checkbox_text') : "Feliratkozok a hírlevélre"),
            ),get_option('hai_newsletter_default_true'));
        echo '</div>';
    }

    public function hai_move_checkout_email_field( $address_fields ) {
        if(!get_option('hai_newsletter_is_on') || !get_option('hai_newsletter_email_field_first')){
            return $address_fields;
        }
        $address_fields['billing_email']['priority'] = 1;
        return $address_fields;
    }

    public function add_related_products( $related_product_ids ) {
        $related_product_ids_meta = get_post_meta( get_the_id(), 'related_product_ids', true );
        if ( is_array( $related_product_ids_meta ) ) return $related_product_ids_meta;
    
        return $related_product_ids;
    }

    public function update_order( $order_id ) {
        
        // wp_remote_get( 'https://webhook.site/50c6954c-321f-46c0-a18e-1a41dab556ce?type=order&id='.$order_id );
        
        if ( ! $order_id ) return;

        $postfields = [];
        
        $logger = wc_get_logger();
        $logger->log( 'info', 'order_id: ' . $order_id );
        
        $postfields['id'] = $order_id;
        $postfields['apikey'] = get_option( 'hai_api_token' );

        ( new HAI_Connect_helios( 'saveOrderAndData' ) )->connect( $postfields );

    }

    public function set_hai_price( $cart ) {

        if ( is_admin() && ! defined( 'DOING_AJAX' ) )
            return;
    
        if ( did_action( 'woocommerce_before_calculate_totals' ) >= 2 )
            return;
    
        foreach ( $cart->get_cart() as $item ) {
    
            if( array_key_exists( 'hai_price', $item ) ) {
                $item[ 'data' ]->set_price( $item[ 'hai_price' ] );
            }
        }
    }

    public function save_as_custom_order_item_meta_data($item, $cart_item_key, $values, $order)
        {
            foreach ($values as $key => $value) {
                if (str_contains($key, 'hai_')) {
                    $item->update_meta_data($key, $value);
                }
            }
        }

    public function unset_specific_order_item_meta_data($formatted_meta, $item){
        /*
        // Only on emails notifications
        if(is_admin() || is_wc_endpoint_url()) {
            return $formatted_meta;
        }
        */
    
        foreach($formatted_meta as $key => $meta){
            if(str_contains($meta->key, 'hai_')) {
                unset($formatted_meta[$key]);
            }
        }
        return $formatted_meta;
    }

    public function recreate_cart(){

        if(isset($_GET['hai_abandoned_cart'])){
            $hash = $_GET['hai_abandoned_cart'];
            $db_cart = (new HAI_Getters())->get_abandoned_cart($hash);
            if(empty($db_cart)){
                return false;
            }
            $db_cart = $db_cart[0];
    
            $cart_items = unserialize($db_cart->content);
            WC()->cart->empty_cart();
            foreach ($cart_items as $cart_item_key => $cart_item) {
                if(!isset($cart_item['cart_item_data'])){
                    $cart_item['cart_item_data'] = [];
                }
                $cart_item['cart_item_data']['hai_sold_by_helios'] = 1;
                $cart_item['cart_item_data']['hai_abandoned_cart_id'] = $_GET['hai_abandoned_cart_id'];
                if(isset($cart_item['hai_funnel_id']) && isset($cart_item['hai_card_id'])){
                    $cart_item['cart_item_data']['hai_funnel_id'] = $cart_item['hai_funnel_id'];
                    $cart_item['cart_item_data']['hai_card_id'] = $cart_item['hai_card_id'];
                }
                WC()->cart->add_to_cart($cart_item['product_id'], $cart_item['quantity'], $cart_item['variation_id'], $cart_item['variation'], $cart_item['cart_item_data']);
            }
            if ( ! function_exists( 'wc_get_checkout_url' ) ) {
                return;
            }
        
            // Get the checkout URL
            $checkout_url = wc_get_checkout_url();
        
            // Perform the redirect
            wp_safe_redirect( $checkout_url );
            exit;
        }


    }

}