<?php

namespace HAI\Includes;

use WC_Product_Query;

class HAI_Getters {

    public function __construct() {
        global $wpdb;
        $this->wpdb = $wpdb;

        $this->hai_funnels = $this->wpdb->prefix . "hai_funnels";
        $this->hai_funnel_views = $this->wpdb->prefix . "hai_funnel_views";
        $this->hai_product_views = $this->wpdb->prefix . "hai_product_page_visit_log";
        $this->hai_funnel_clicks = $this->wpdb->prefix . "hai_funnel_clicks";
        $this->hai_abandoned_carts = $this->wpdb->prefix . "hai_abandoned_carts";
    }

    public function get_triggered_funnels( $id ) {
        return $this->wpdb->get_results("SELECT * FROM {$this->hai_funnels} WHERE trigger_product_ids LIKE '%\"$id\"%' AND active = 1");
    }

    public function get_funnel( $funnel_id, $card_id ){
        return $this->wpdb->get_results("SELECT * FROM {$this->hai_funnels} WHERE funnel_id = $funnel_id AND card_id = $card_id");
    }

    public function all_shipping_methods() {

        $zone_ids = array_keys( array('') + \WC_Shipping_Zones::get_zones() );
        
        $shipping_methods_arr = [];
        
        foreach ( $zone_ids as $zone_id ) {
            $shipping_zone = new \WC_Shipping_Zone($zone_id);
        
            $shipping_methods = $shipping_zone->get_shipping_methods( true, 'values' );
        
            foreach ( $shipping_methods as $instance_id => $shipping_method ) {
                // echo '<pre>';
                // var_dump($shipping_method);
                // echo '</pre>';
                $details = [
                    'id' => $shipping_method->id,
                    'instance_id' => $shipping_method->instance_id,
                    'title' => $shipping_method->title,
                    'method_title' => $shipping_method->method_title,
                    'method_description' => $shipping_method->method_description,
                    'enabled' => $shipping_method->enabled,
                    'cost' => ( isset( $shipping_method->cost ) ) ? $shipping_method->cost : '',
                    'tax_status' => $shipping_method->tax_status,
                ];
                $shipping_methods_arr[] = $details;
            }
        }

        return $shipping_methods_arr;

    }

    public function products_ids() {

        $ids = wc_get_products( array( 'return' => 'ids', 'limit' => -1, 'type' => [ 'simple', 'variable', 'variation' ] ) );

        return $ids;

    }

    public function products( $page, $metas ) {

        $posts_per_page = 100;

        if ( $metas ) {
            $metas = explode( '|', $metas );
        }

        $args = array(
            'post_type' => 'product',
            'posts_per_page' => $posts_per_page,
            'type' => [ 'simple', 'variable', 'variation' ],
            'post_status' => 'publish',
            'orderby' => 'ID',
            'order' => 'ASC',
            'paged' => $page,
        );

        $products = wc_get_products( $args );

        $product_datas = [];

        foreach ( $products as $product ) {

            $product_datas[] = $this->product_data( $product, $metas );

        }

        return $product_datas;

    }

    public function products_count() {

        global $wpdb;
        $count_product = $wpdb->get_var("SELECT COUNT(*) FROM $wpdb->posts WHERE `post_type` LIKE 'product' AND post_status='publish'");
        $count_product_variation = $wpdb->get_var("SELECT COUNT(*) FROM $wpdb->posts WHERE `post_type` LIKE 'product_variation' AND post_status='publish'");

        return $count_product + $count_product_variation;

    }

    public function product( $product_id, $metas ) {

        $product = wc_get_product( $product_id );

        if ( ! $product ) return 'Wrong product id!';

        return $this->product_data( $product, $metas );;

    }

    public function order_statuses_count() {

        $order_statuses = wc_get_order_statuses();

        $orders_count = [];

        foreach ( $order_statuses as $key => $order_status ) {
            $orders_count[$key] = wc_orders_count( $key );
        }

        return $orders_count;

    }

    public function orders_count() {

        $args = array(
            'orderby' => 'date',
            'order' => 'ASC',
            'return' => 'ids',
        );

        $orders = wc_get_orders( $args );

        return count( $orders );

    }

    public function orders( $page = 1 ) {

        $args = array(
            'orderby' => 'date',
            'order' => 'ASC',
            'return' => 'ids',
            'posts_per_page' => 100,
            'page' => $page,
        );

        $orders = wc_get_orders( $args );

        $orders_datas = [];

        foreach ( $orders as $order_id ) {
            $orders_datas[] = $this->order( $order_id );
        }

        return $orders_datas;

    }

    public function order( $order_id ) {

        $order = wc_get_order($order_id);

        $order_data = $order->get_data();

        $items = $order->get_items();
        $line_items = [];
        foreach ( $items as $item_id => $item ) {
            $product = $item->get_product();
            $line_items[] = $item->get_data();
        }
        
        $order_data['line_items'] = $line_items;

        $taxes = $order->get_taxes();
        $tax_lines = [];
        foreach ( $taxes as $tax_id => $tax ) {
            $tax_lines[] = $tax->get_data();
        }
        
        $order_data['tax_lines'] = $tax_lines;

        $shippings = $order->get_items( 'shipping' );
        $shipping_lines = [];
        foreach ( $shippings as $shipping_id => $shipping ) {
            $shipping_lines[] = $shipping->get_data();
        }
        
        $order_data['shipping_lines'] = $shipping_lines;

        $fees = $order->get_items( 'fee' );
        $fee_lines = [];
        foreach ( $fees as $fee_id => $fee ) {
            $fee_lines[] = $fee->get_data();
        }
        
        $order_data['fee_lines'] = $fee_lines;

        $coupons = $order->get_items( 'coupon' );
        $coupon_lines = [];
        foreach ( $coupons as $coupon_id => $coupon ) {
            $coupon_lines[] = $coupon->get_data();
        }
        
        $order_data['coupon_lines'] = $coupon_lines;

        return json_decode(json_encode( $order_data ) , true);

    }

    public function all_categories() {

        $taxonomy     = 'product_cat';
        $orderby      = 'name';  
        $show_count   = 1;      // 1 for yes, 0 for no
        $pad_counts   = 1;      // 1 for yes, 0 for no
        $hierarchical = 1;      // 1 for yes, 0 for no  
        $title        = '';  
        $empty        = 0;
    
        $args = array(
            'taxonomy'     => $taxonomy,
            'orderby'      => $orderby,
            'show_count'   => $show_count,
            'pad_counts'   => $pad_counts,
            'hierarchical' => $hierarchical,
            'title_li'     => $title,
            'hide_empty'   => $empty
        );
        $all_categories = get_categories( $args );

        $categories = [];

        foreach ( $all_categories as $category ) {

            $category = json_decode(json_encode( $category ), true);

            $category['thumbnail'] = '';
            if ( $thumbnail_id = get_woocommerce_term_meta( $category['term_id'], 'thumbnail_id', true ) ) {
                $category['thumbnail'] = wp_get_attachment_url( $thumbnail_id );
            }

            $categories[] = $category;

        }

        return $categories;
    
    }

    public function users($page = 1) {
        $users_data = array();
    
        $args = array(
            'number' => 100,
            'paged' => $page,
        );
    
        $user_query = new \WP_User_Query($args);
        $users = $user_query->get_results();
    
        foreach ($users as $user) {
            $user_data = get_userdata($user->ID);

            $user_roles = $user->roles;
    
            $users_data[] = array(
                'user_id' => $user->ID,
                'user_roles' => $user_roles,
                'user_login' => $user_data->user_login,
                'user_email' => $user_data->user_email,
            );
        }
    
        return $users_data;
    }

    public function all_tags() {
        $tags = get_terms(array(
            'taxonomy' => 'product_tag',
            'hide_empty' => false,
        ));
    
        return $tags;
    }

    public function all_attributes() {
        $attributes = wc_get_attribute_taxonomies();
        $data = array();
    
        foreach ($attributes as $attribute) {
            $attribute_name = 'pa_' . $attribute->attribute_name;
            $terms = get_terms(array(
                'taxonomy' => $attribute_name,
                'hide_empty' => false,
            ));
    
            $attribute_data = array(
                'id' => $attribute->attribute_id,
                'slug' => $attribute->attribute_name,
                'name' => $attribute->attribute_label,
                'values' => array(),
            );

            if (!empty($terms)) {
                foreach ($terms as $term) {
                    $attribute_data['values'][] = array(
                        'id' => $term->term_id,
                        'slug' => $term->slug,
                        'name' => $term->name,
                    );
                }
            }
            
            $data[] = $attribute_data;
        }
    
        return $data;
    }

    public function product_metas() {

        $meta_keys = [];
        
        $args = array(
            'post_type'      => 'product', // Csak a termékek
            'posts_per_page' => -1, // Az összes termék
        );
        
        $products = new \WP_Query( $args );
        
        if ( $products->have_posts() ) {
            while ( $products->have_posts() ) {
                $products->the_post();
        
                $product_id = get_the_ID();
                $product_meta = get_post_meta( $product_id );
        
                foreach ( $product_meta as $meta_key => $meta_value ) {
                    
                    // if ( strpos( $meta_key, '_' ) === 0 ) {
                    //     continue;
                    // }
        
                    if ( ! in_array( $meta_key, $meta_keys ) ) $meta_keys[] = $meta_key;
                }
            }
        }
        
        wp_reset_postdata();

        return $meta_keys;

    }

    private function product_data( $product, $metas ) {

        if ( $product->get_type() != 'variation' ) {
        
            $attributes = [];
            
            foreach ( $product->get_attributes() as $key => $value ) {

                $values = [];

                foreach ( $value->get_options() as $v ) {

                    ( is_numeric( $v ) ) ? $values[] = get_term_by( 'term_taxonomy_id', $v )->name : $values[] = $v;

                }
            
                // $attributes[$key] = $value->get_options();
                $attributes[] = [
                    'id' => $value->get_id(),
                    'slug' => $value->get_name(),
                    'values' => $values,
                ];
            
            }
        
        } else {
            // wc_attribute_taxonomy_id_by_name($taxonomy_name);
            // $attributes = $product->get_attributes();
        
            $attributes = [];
            
            foreach ( $product->get_attributes() as $key => $value ) {
            
                // $attributes[$key] = $value->get_options();
                $attributes[] = [
                    'id' => wc_attribute_taxonomy_id_by_name($key),
                    'slug' => $key,
                    'values' => [$value],
                ];
            
            }
        
        }

        $tax = new \WC_Tax();
        
        $taxes = $tax->get_rates($product->get_tax_class());
        if ( count( $taxes ) > 0 ) {
            $rates = array_shift($taxes);
            $item_rate = round(array_shift($rates));
        } else $item_rate = 0;

        $product_data = array(
            'id' => $product->get_id(),
            'parent_id' => $product->get_parent_id(),
            'type' => $product->get_type(),
            'name' => $product->get_name(),
            'slug' => $product->get_slug(),
            'permalink' => $product->get_permalink(),
            'sku' => $product->get_sku(),
            'status' => $product->get_status(),
            'backorders' => $product->get_backorders(),
            'stock_quantity' => ( $product->get_stock_quantity() < 0 ) ? 0 : $product->get_stock_quantity(),
            'stock_status' => $product->get_stock_status(),
            'length' => $product->get_length(),
            'width' => $product->get_width(),
            'height' => $product->get_height(),
            'weight' => $product->get_weight(),
            'tax' => $item_rate,
            'price' => $product->get_price(),
            'regular_price' => $product->get_regular_price(),
            'sale_price' => $product->get_sale_price(),
            'date_on_sale_from' => $product->get_date_on_sale_from(),
            'date_on_sale_to' => $product->get_date_on_sale_to(),
            'description' => $product->get_description(),
            'short_description' => $product->get_short_description(),
            'cross_sell_ids' => $product->get_cross_sell_ids(),
            'upsell_ids' => $product->get_upsell_ids(),
            'related_product_ids' => get_post_meta( $product->get_id(), 'related_product_ids', true ),
            'attributes' => $attributes,
            'categories' => wp_get_post_terms( $product->get_id(), 'product_cat' ),
            'tags' => wp_get_post_terms( $product->get_id(), 'product_tag' ),
        );

        $product_data['thumbnail'] = '';
        if ( isset( wp_get_attachment_image_src(get_post_thumbnail_id($product->get_id()), 'single-post-thumbnail')[0] ) ) {
            $product_data['thumbnail'] = wp_get_attachment_image_src(get_post_thumbnail_id($product->get_id()), 'single-post-thumbnail')[0];
        }

        $attachment_ids = $product->get_gallery_image_ids();
        $product_data['gallery'] = [];
        foreach( $attachment_ids as $attachment_id ) {
            $product_data['gallery'][] = wp_get_attachment_url( $attachment_id );
        }

        if ( $metas ) {

            $metas = explode( '|', $metas );

            foreach ( $metas as $meta ) {
                $product_data['metas'][$meta] = get_post_meta( $product->get_id(), $meta, true );
            }

        }

        return $product_data;
        
    }

    public function all_customers() {
        
        $orders = wc_get_orders( array( 'limit' => -1 ) );

        if ( $orders ) {

            $customers = [];
    
            foreach ( $orders as $order ) {

                $order_data = $order->get_data();
                
                $customer_data['user_id'] = $order_data['customer_id'];
                
                $customer_data['billing'] = $order_data['billing'];

                $customer_data['shipping'] = $order_data['shipping'];

                $customers[$order_data['billing']['email']][] = $customer_data;

            }

        } else {
            echo '<p>Nincsenek tranzakciók.</p>';
        }

        return $customers;

    }

    public function get_abandoned_cart($hash){
        return $this->wpdb->get_results("SELECT * FROM {$this->hai_abandoned_carts} WHERE `hash` like '$hash'");
    }

    public function get_funnel_views(){
        return $this->wpdb->get_results("SELECT * FROM {$this->hai_funnel_views}");
    }

    public function delete_funnel_views($timestamp){
        return $this->wpdb->query( 
            $this->wpdb->prepare( 
                "DELETE FROM $this->hai_funnel_views WHERE timestamp <= %d",
                $timestamp
            )
        );
    }

    public function get_product_views(){
        return $this->wpdb->get_results("SELECT * FROM {$this->hai_product_views}");
    }

    public function delete_product_views($timestamp){
        return $this->wpdb->query( 
            $this->wpdb->prepare( 
                "DELETE FROM $this->hai_product_views WHERE timestamp <= %d",
                $timestamp
            )
        );
    }

    public function get_funnel_clicks(){
        return $this->wpdb->get_results("SELECT * FROM {$this->hai_funnel_clicks}");
    }

    public function delete_funnel_clicks($timestamp){
        return $this->wpdb->query( 
            $this->wpdb->prepare( 
                "DELETE FROM $this->hai_funnel_clicks WHERE timestamp <= %d",
                $timestamp
            )
        );
    }

}