<?php

namespace HAI\Includes;

use WP_REST_Controller;
use HAI\Includes\HAI_Getters;
use HAI\Includes\HAI_Setters;
// use HAI\Includes\HAI_Submits;

class HAI_Settings_Route extends WP_REST_Controller {

    protected $namespace;
    protected $token;

    public function __construct() {
        $this->namespace = 'heliosai/v1';
        // $this->token = 'Mg7Le5OCOeJjv0r2N35HHf9GxZjdP5';
        $this->token = get_option('hai_api_token');
    }

    /**
     * Register Routes
     */
    public function register_routes() {
        register_rest_route(
            $this->namespace,
            '/hai',
            [
                [
                    'methods'             => \WP_REST_Server::READABLE,
                    'callback'            => [ $this, 'get_hai_items' ],
                    'permission_callback' => [ $this, 'get_permission_check' ],
                    'args'                => $this->get_collection_params()
                ],
                [
                    'methods'             => \WP_REST_Server::CREATABLE,
                    'callback'            => [ $this, 'create_hai_items' ],
                    'permission_callback' => [ $this, 'create_items_permission_check' ],
                    'args'                => $this->get_endpoint_args_for_item_schema( true )
                ]
            ]
        );
        register_rest_route(
            $this->namespace,
            '/token',
            [
                [
                    'methods'             => \WP_REST_Server::CREATABLE,
                    'callback'            => [ $this, 'create_token' ],
                    'permission_callback' => [ $this, 'create_token_permission_check' ],
                    'args'                => $this->get_endpoint_args_for_item_schema( true )
                ]
            ]
        );
    }

    /**
     * Get items response
     */
    public function get_hai_items( $request ) {
        
        if ( isset( $request['type'] ) ) {

            switch ( $request['type'] ) {

                case 'start_sync':
                    $result = [];
                    $result['products'] = ( new HAI_Getters() )->get_products_count();
                    $result['users'] = count_users()['total_users'];
                    return rest_ensure_response( $result );

                case 'products':
                    $result = ( new HAI_Getters() )->products( $request['page'], ( isset( $request['metas'] ) ) ? $request['metas'] : null );
                    return rest_ensure_response( $result );

                case 'products_count':
                    $result = ( new HAI_Getters() )->products_count();
                    return rest_ensure_response( $result );

                case 'users':
                    $result = ( new HAI_Getters() )->users( $request['page'] );
                    return rest_ensure_response( $result );

                case 'all_customers':
                    $result = ( new HAI_Getters() )->all_customers( $request['page'] );
                    return rest_ensure_response( $result );

                case 'products_ids':
                    $result = ( new HAI_Getters() )->products_ids();
                    return rest_ensure_response( $result );

                case 'product':
                    $result = ( new HAI_Getters() )->product( $request['id'], ( isset( $request['metas'] ) ) ? $request['metas'] : null );
                    return rest_ensure_response( $result );

                case 'product_metas':
                    $result = ( new HAI_Getters() )->product_metas();
                    return rest_ensure_response( $result );

                case 'order_statuses_count':
                    $result = ( new HAI_Getters() )->order_statuses_count();
                    return rest_ensure_response( $result );

                case 'orders_count':
                    $result = ( new HAI_Getters() )->orders_count();
                    return rest_ensure_response( $result );

                case 'orders':
                    $result = ( new HAI_Getters() )->orders( $request['page'] );
                    return rest_ensure_response( $result );

                case 'order':
                    $result = ( new HAI_Getters() )->order( $request['id'] );
                    return rest_ensure_response( $result );

                case 'all_categories':
                    $result = ( new HAI_Getters() )->all_categories();
                    return rest_ensure_response( $result );

                case 'all_tags':
                    $result = ( new HAI_Getters() )->all_tags();
                    return rest_ensure_response( $result );

                case 'all_attributes':
                    $result = ( new HAI_Getters() )->all_attributes();
                    return rest_ensure_response( $result );

                case 'all_shipping_methods':
                    $result = ( new HAI_Getters() )->all_shipping_methods();
                    return rest_ensure_response( $result );

                case 'all_payment_methods':
                    return rest_ensure_response( WC()->payment_gateways()->payment_gateways() );

                case 'funnel-views':
                    $result = ( new HAI_Getters() )->get_funnel_views();
                    return rest_ensure_response( $result );

                case 'delete-funnel-views':
                    $result = ( new HAI_Getters() )->delete_funnel_views($_GET['timestamp']);
                    return rest_ensure_response( $result );

                case 'product-views':
                    $result = ( new HAI_Getters() )->get_product_views();
                    return rest_ensure_response( $result );

                case 'delete-product-views':
                    $result = ( new HAI_Getters() )->delete_product_views($_GET['timestamp']);
                    return rest_ensure_response( $result );
                    
                case 'funnel-clicks':
                    $result = ( new HAI_Getters() )->get_funnel_clicks();
                    return rest_ensure_response( $result );
                    
                case 'delete-funnel-clicks':
                    $result = ( new HAI_Getters() )->delete_funnel_clicks($_GET['timestamp']);
                    return rest_ensure_response( $result );

                default:
                return 'ERROR : Undefined type!';

            }

        }

        // $all_headers = $request->get_headers();
        
        // return $all_headers;
        
        return 'ERROR : Missing type!';
    }

    /**
     * Get items permission check
     */
    public function get_permission_check( $request ) {
        // if( current_user_can( 'administrator' ) ) {
        //     return true;
        // }

        $all_headers = $request->get_headers();

        if ( explode( 'Bearer ', $all_headers['authorization'][0] )[1] === $this->token ) {
            return true;
        }
        return false;
        
    }

    /**
     * Create items response
     */
    public function create_token( $request ) {
        
        if ( isset( $request['token'] ) ) {

            if ( $this->token && $request['token'] === $this->token ) {
                return 'OK';
            } elseif ( $this->token && $request['token'] !== $this->token ) {
                return 'Wrong token!';
            }

            update_option('hai_api_token', $request['token']);

            return 'Token updated';

        }
        
        return 'ERROR : Missing token!';
    }

    /**
     * Create items response
     */
    public function create_hai_items( $request ) {
        
        if ( isset( $request['type'] ) ) {

            switch ( $request['type'] ) {
                
                case 'order':
                    $order = json_decode($request['order'], true);
                    $result = ( new HAI_Setters )->order( $order );
                    return rest_ensure_response( $result );
                
                case 'order_status':
                    $result = ( new HAI_Setters )->order_status( $request['id'], $request['status'] );
                    return rest_ensure_response( $result );
                
                case 'create_category':
                    $result = ( new HAI_Setters )->create_category( $request['name'], ( isset( $request['description'] ) ) ? $request['description'] : '', ( isset( $request['parent_id'] ) ) ? $request['parent_id'] : false );
                    return rest_ensure_response( $result );
                
                case 'create_tag':
                    $result = ( new HAI_Setters )->create_tag( $request['name'], ( isset( $request['description'] ) ) ? $request['description'] : '', ( isset( $request['parent_id'] ) ) ? $request['parent_id'] : false );
                    return rest_ensure_response( $result );
                
                case 'create_attribute':
                    $result = ( new HAI_Setters )->create_attribute( $request['name'] );
                    return rest_ensure_response( $result );
                
                case 'orders':
                    $orders = json_decode($request['orders'], true);
                    $result = ( new HAI_Setters )->orders( $orders );
                    return rest_ensure_response( $result );
                
                case 'categories':
                    $categories = json_decode($request['categories'], true);
                    $result = ( new HAI_Setters )->categories( $categories );
                    return rest_ensure_response( $result );
                
                case 'tags':
                    $tags = json_decode($request['tags'], true);
                    $result = ( new HAI_Setters )->tags( $tags );
                    return rest_ensure_response( $result );
                
                case 'products':
                    $products = json_decode($request['products'], true);
                    $result = ( new HAI_Setters )->products( $products );
                    return rest_ensure_response( $result );

                case 'attributes':
                    $attributes = json_decode($request['attributes'], true);
                    $result = ( new HAI_Setters )->attributes( $attributes );
                    return rest_ensure_response( $result );

                case 'funnels':
                    $funnels = json_decode($request['funnels'], true);
                    // return $funnels;
                    $result = ( new HAI_Setters )->funnels( $funnels );
                    return $result;
                
                default:
                    return 'ERROR : Undefined type!';

            }

        }
        
        return 'ERROR : Missing type!';
    }

    /**
     * Create item permission check
     */
    public function create_items_permission_check( $request ) {

        $all_headers = $request->get_headers();

        if ( explode( 'Bearer ', $all_headers['authorization'][0] )[1] === $this->token ) {
            return true;
        }
        return false;

    }

    /**
     * Create item permission check
     */
    public function create_token_permission_check( $request ) {
        
        // $all_headers = $request->get_headers();

        // if ( $this->token ) {
        //     return true;
        // }
        return true;

    }

    /**
     * Retrives the query parameters for the items collection
     */
    public function get_collection_params() {
        return [];
    }

}