<?php

/**
 *
 * @since       1.0.0
 *
 * Plugin Name: Helios AI Connect
 * Version: 1.1.30
 * Description: A Helios AI professzionális kereskedelmi szoftveréhez készült wordpress integrációs bővítmény
 * Author: Helios AI
 * Author URI: https://heliosai.hu/wordpress
 * Text Domain: heliosai-connect
 */

if (!defined('ABSPATH')) exit();

require_once __DIR__ . '/vendor/autoload.php';

require dirname(__FILE__) . '/plugin-update-checker/plugin-update-checker.php';
use HAI\Includes\HAI_Api;
use HAI\Includes\HAI_Dbs;
use HAI\Includes\HAI_Admin;
use HAI\Includes\HAI_Submits;
use HAI\Includes\HAI_Functions;
use HAI\Includes\HAI_Elementor_Form_Register;

use HAI\Modules\HAI_Funnels;

use YahnisElsts\PluginUpdateChecker\v5p0\PucFactory;

// use HAI\Includes\HAI_Submits;

final class WP_HAI_Or {

    /**
     * Define Plugin Version
     */
    const VERSION = '1.1.30';

    /**
     * Construct Function
     * @since 1.0.0
     */
    public function __construct() {
        $this->plugin_constants();
        register_activation_hook(__FILE__, [$this, 'activate']);
        register_deactivation_hook(__FILE__, [$this, 'deactivate']);
        add_action('plugins_loaded', [$this, 'init_plugin']);
        add_action('plugins_loaded', [$this, 'check_for_update']);
        add_action('wp_enqueue_scripts', [$this, 'hai_script_reg']);
        //add_action('elementor/editor/after_enqueue_scripts', [$this,'elementor_enqueue_scripts']);
    }

    /**
     * Plugin Constants
     * @since 1.0.0
     */
    public function plugin_constants() {
        define('HAI_VERSION', self::VERSION);
        define('HAI_PLUGIN_PATH', trailingslashit(plugin_dir_path(__FILE__)));
        define('HAI_PLUGIN_URL', trailingslashit(plugins_url('', __FILE__)));
		if(str_contains(site_url(), 'heliosai.onlinevagyok.com')){
			define('HAI_ADMINURL_PREFIX', 'dev');
		}else{
			define('HAI_ADMINURL_PREFIX', 'admin');
		}
        define('HAI_NONCE', 'b?le*;K7.T2jk_*(+3&[G[xAc8O~Fv)2T/Zk9N:GKBkn$piN0.N%N~X91VbCn@.4');
    }

    /**
     * Singletone Instance
     * @since 1.0.0
     */
    public static function init() {

        $myUpdateChecker = PucFactory::buildUpdateChecker(
            "http://dev.heliosai.eu/upload/wpplugin/heliosai-connect.json",
            __FILE__,
            'heliosai-connect'
        );

        $args = array(
            'type' => 'array',
        );

        register_setting('hai-settings', 'hai_settings', $args);

        static $instance = false;

        if (!$instance) {
            $instance = new self();
        }

        return $instance;
    }

    /**
     * On Plugin Activation
     * @since 1.0.0
     */
    public function activate() {
        // On plugin activation
        $is_installed = get_option('hai_is_installed');

        if (!$is_installed) {
            update_option('hai_is_installed', time());
        }

        update_option('hai_is_installed', HAI_VERSION);

        new HAI_Dbs();

    }

    /**
     * On Plugin De-actiavtion
     * @since 1.0.0
     */
    public function deactivate() {
        // On plugin deactivation
    }

    public function hai_script_reg() {
        wp_enqueue_style('hai-style', plugin_dir_url( __FILE__ ) . 'src/css/style.css');
        wp_enqueue_script('jquery');
        wp_enqueue_script( 'wc-cart-fragments' );
        wp_enqueue_script('hai-script', plugins_url('src/js/script.js', __FILE__), array('jquery'), self::VERSION, true);
        $data = array(
            'upload_url' => admin_url('async-upload.php'),
            'ajax_url' => admin_url('admin-ajax.php'),
            'nonce' => wp_create_nonce('media-form'),
        );
        wp_localize_script('hai-script', 'hai_ajax_object', $data);
        // enqueue the file into the wp website.
        wp_enqueue_script( 'hai-wc-price-js', plugins_url('src/js/wc_price.js', __FILE__), array( 'jquery' ), '1.0', false );

        // add inline script to pass currency informations from php to the js.
        $wc_store_object = array(
        'html' => true,
        'currency_symbol' => get_woocommerce_currency_symbol( get_woocommerce_currency() ),
        'currency_position' => get_option( 'woocommerce_currency_pos', true ),
        'decimal_separator' => wc_get_price_decimal_separator(),
        'currency_format_trim_zeros' => wc_get_price_thousand_separator(),
        'currency_format_num_decimals' => wc_get_price_decimals(),
        'price_format' => get_woocommerce_price_format(),
        );
        wp_add_inline_script( 'hai-wc-price-js', ' var hai_wc_settings_args=' . wp_json_encode( $wc_store_object ) . ';' );

    }

    /**
     * Check for plugin update and run function if updated
     * @since 1.1.20
     */
    public function check_for_update() {
        $current_version = get_option('hai_plugin_version');
        
        if ($current_version !== self::VERSION) {
            $this->run_on_update();
            update_option('hai_plugin_version', self::VERSION);
        }
    }

    /**
     * Function to run on plugin update
     * @since 1.1.20
     */
    public function run_on_update() {
        // Code to execute on plugin update
        new HAI_Dbs();
        error_log('Helios AI Connect plugin has been updated to version ' . self::VERSION);
        // Add your custom update code here
    }

    /**
     * Init Plugin
     * @since 1.0.0
     */
    public function init_plugin() {
        // init
        global $hai_submits;
        global $hai_funnel;
        new HAI_Api();
        new HAI_Admin();
        new HAI_Functions();
        $hai_submits = new HAI_Submits();
        $hai_funnel = new HAI_Funnels();
        new HAI_Elementor_Form_Register();
    }

    /*public function elementor_enqueue_scripts() {
    // Only load the script in the Elementor editor
        if (\Elementor\Plugin::instance()->editor->is_edit_mode()) {
            wp_enqueue_script(
                'hai-elementor',
                plugins_url('src/js/elementor.js', __FILE__),
                ['jquery'],
                '1.0.0',
                true
            );
            $data = array(
                'upload_url' => admin_url('async-upload.php'),
                'ajax_url' => admin_url('admin-ajax.php'),
                'nonce' => wp_create_nonce('media-form'),
            );
            wp_localize_script('hai-elementor', 'hai_ajax_object', $data);
        }
    }*/
}

function wp_hai_or() {
    return WP_HAI_Or::init();
}

wp_hai_or();

if (!function_exists('log_this')) {
    function log_this($message, $file_name = "log"){
        if(gettype($message) != "string"){
            $message = json_encode($message);
        }
        file_put_contents(plugin_dir_path( __FILE__ )."/".$file_name.".txt", date("Y-m-d H:i:s") . " - " . $message . PHP_EOL, FILE_APPEND);
    }
}